@@ -24,15 +24,16 @@ from aodhclient import exceptions
 from aodhclient.i18n import _
 from aodhclient import utils
 
-ALARM_TYPES = ['event', 'composite',
+ALARM_TYPES = ['prometheus', 'event', 'composite', 'threshold',
                'gnocchi_resources_threshold',
                'gnocchi_aggregation_by_metrics_threshold',
-               'gnocchi_aggregation_by_resources_threshold']
+               'gnocchi_aggregation_by_resources_threshold',
+               'loadbalancer_member_health']
 ALARM_STATES = ['ok', 'alarm', 'insufficient data']
 ALARM_SEVERITY = ['low', 'moderate', 'critical']
 ALARM_OPERATORS = ['lt', 'le', 'eq', 'ne', 'ge', 'gt']
 ALARM_OP_MAP = dict(zip(ALARM_OPERATORS, ('<', '<=', '=', '!=', '>=', '>')))
-
+STATISTICS = ['max', 'min', 'avg', 'sum', 'count']
 ALARM_LIST_COLS = ['alarm_id', 'type', 'name', 'state', 'severity', 'enabled']
 
 
@@ -102,7 +103,7 @@ def _format_alarm(alarm):
         alarm["time_constraints"] = jsonutils.dumps(alarm["time_constraints"],
                                                     sort_keys=True,
                                                     indent=2)
-    # only works for event alarm
+    # only works for threshold and event alarm
     if isinstance(alarm.get('query'), list):
         query_rows = []
         for q in alarm['query']:
@@ -264,13 +265,15 @@ class CliAlarmCreate(show.ShowOne):
         common_group = parser.add_argument_group('common alarm rules')
         common_group.add_argument(
             '--query', metavar='<QUERY>', dest='query',
-            help="For alarms of type event: "
+            help="For alarms of type threshold or event: "
                  "key[op]data_type::value; list. data_type is optional, "
                  "but if supplied must be string, integer, float, or boolean. "
                  'For alarms of '
                  'type gnocchi_aggregation_by_resources_threshold: '
                  'need to specify a complex query json string, like:'
-                 ' {"and": [{"=": {"ended_at": null}}, ...]}.')
+                 ' {"and": [{"=": {"ended_at": null}}, ...]}. '
+                 'For alarms of type prometheus this should be valid '
+                 'PromQL query.')
         common_group.add_argument(
             '--comparison-operator', metavar='<OPERATOR>',
             dest='comparison_operator', choices=ALARM_OPERATORS,
@@ -283,11 +286,26 @@ class CliAlarmCreate(show.ShowOne):
             '--threshold', type=float, metavar='<THRESHOLD>',
             dest='threshold', help='Threshold to evaluate against.')
 
+        # For event type alarm
         event_group = parser.add_argument_group('event alarm')
         event_group.add_argument(
             '--event-type', metavar='<EVENT_TYPE>',
             dest='event_type', help='Event type to evaluate against')
 
+        # For Ceilometer threshold type alarm
+        threshold_group = parser.add_argument_group('threshold alarm')
+        threshold_group.add_argument(
+            '-m', '--meter-name', metavar='<METER NAME>',
+            dest='meter_name', help='Meter to evaluate against')
+        threshold_group.add_argument(
+            '--period', type=int, metavar='<PERIOD>', dest='period',
+            help='Length of each period (seconds) to evaluate over.')
+        threshold_group.add_argument(
+            '--statistic', metavar='<STATISTIC>', dest='statistic',
+            choices=STATISTICS,
+            help='Statistic to evaluate, one of: ' + str(STATISTICS))
+
+        # For common Gnocchi threshold type alarm
         gnocchi_common_group = parser.add_argument_group(
             'common gnocchi alarm rules')
         gnocchi_common_group.add_argument(
@@ -312,16 +330,46 @@ class CliAlarmCreate(show.ShowOne):
             '--resource-id', metavar='<RESOURCE_ID>',
             dest='resource_id', help='The id of a resource.')
 
+        # For composite type alarm
         composite_group = parser.add_argument_group('composite alarm')
         composite_group.add_argument(
             '--composite-rule', metavar='<COMPOSITE_RULE>',
             dest='composite_rule',
             type=jsonutils.loads,
             help='Composite threshold rule with JSON format, the form can '
-                 'be a nested dict which combine gnocchi rules by '
+                 'be a nested dict which combine threshold/gnocchi rules by '
                  '"and", "or". For example, the form is like: '
-                 '{"or":[RULE1, RULE2, {"and": [RULE3, RULE4]}]}.'
+                 '{"or":[RULE1, RULE2, {"and": [RULE3, RULE4]}]}, The '
+                 'RULEx can be basic threshold rules but must include a '
+                 '"type" field, like this: {"threshold": 0.8,'
+                 '"meter_name":"cpu_util","type":"threshold"}'
         )
+
+        loadbalancer_member_health_group = parser.add_argument_group(
+            'loadbalancer member health alarm')
+        loadbalancer_member_health_group.add_argument(
+            '--stack-id', metavar='<STACK_NAME_OR_ID>',
+            dest="stack_id",
+            type=str, help=(
+                "Name or ID of the root / top level Heat stack containing the "
+                "loadbalancer pool and members. An update will be triggered "
+                "on the root Stack if an unhealthy member is detected in the "
+                "loadbalancer pool.")
+        )
+        loadbalancer_member_health_group.add_argument(
+            '--pool-id', metavar='<LOADBALANCER_POOL_NAME_OR_ID>',
+            dest="pool_id",
+            type=str, help=(
+                "Name or ID of the loadbalancer pool for which the health of "
+                "each member will be evaluated.")
+        )
+        loadbalancer_member_health_group.add_argument(
+            '--autoscaling-group-id', metavar='<AUTOSCALING_GROUP_NAME_OR_ID>',
+            dest="autoscaling_group_id",
+            type=str, help=(
+                "ID of the Heat autoscaling group that contains the "
+                "loadbalancer members. Unhealthy members will be marked "
+                "as such before an update is triggered on the root stack."))
         self.parser = parser
         return parser
 
@@ -339,7 +387,12 @@ class CliAlarmCreate(show.ShowOne):
             raise argparse.ArgumentTypeError(msg)
 
     def _validate_args(self, parsed_args):
-        if (parsed_args.type == 'gnocchi_resources_threshold' and
+        if (parsed_args.type == 'threshold' and
+                not (parsed_args.meter_name and parsed_args.threshold)):
+            self.parser.error('Threshold alarm requires -m/--meter-name and '
+                              '--threshold parameters. Meter name can be '
+                              'found in Ceilometer')
+        elif (parsed_args.type == 'gnocchi_resources_threshold' and
                 not (parsed_args.metrics and parsed_args.threshold is not None
                      and parsed_args.resource_id and parsed_args.resource_type
                      and parsed_args.aggregation_method)):
@@ -366,17 +419,37 @@ class CliAlarmCreate(show.ShowOne):
               not parsed_args.composite_rule):
             self.parser.error('Composite alarm requires'
                               ' --composite-rule parameter')
+        elif (parsed_args.type == 'loadbalancer_member_health') and \
+                (parsed_args.stack_id is None or
+                 parsed_args.pool_id is None or
+                 parsed_args.autoscaling_group_id is None):
+            self.parser.error('Loadbalancer member health alarm requires'
+                              '--stack-id, --pool-id and'
+                              '--autoscaling-group-id')
+        elif (parsed_args.type == 'prometheus' and
+                not (parsed_args.query and parsed_args.threshold)):
+            self.parser.error('Prometheus alarm requires --query and '
+                              '--threshold parameters.')
 
     def _alarm_from_args(self, parsed_args):
         alarm = utils.dict_from_parsed_args(
-            parsed_args, ['name', 'project_id', 'user_id', 'description',
-                          'state', 'severity', 'enabled', 'alarm_actions',
-                          'ok_actions', 'insufficient_data_actions',
+            parsed_args, ['name', 'type', 'project_id', 'user_id',
+                          'description', 'state', 'severity', 'enabled',
+                          'alarm_actions', 'ok_actions',
+                          'insufficient_data_actions',
                           'time_constraints', 'repeat_actions'])
-        if parsed_args.type == 'event' and parsed_args.query:
+        if parsed_args.type in ('threshold', 'event') and parsed_args.query:
             parsed_args.query = utils.cli_to_array(parsed_args.query)
+        alarm['threshold_rule'] = utils.dict_from_parsed_args(
+            parsed_args, ['meter_name', 'period', 'evaluation_periods',
+                          'statistic', 'comparison_operator', 'threshold',
+                          'query'])
         alarm['event_rule'] = utils.dict_from_parsed_args(
             parsed_args, ['event_type', 'query'])
+        alarm['prometheus_rule'] = (
+            utils.dict_from_parsed_args(parsed_args,
+                                        ['comparison_operator', 'threshold',
+                                         'query']))
         alarm['gnocchi_resources_threshold_rule'] = (
             utils.dict_from_parsed_args(parsed_args,
                                         ['granularity', 'comparison_operator',
@@ -395,6 +468,10 @@ class CliAlarmCreate(show.ShowOne):
                                          'evaluation_periods', 'metric',
                                          'query', 'resource_type']))
 
+        alarm['loadbalancer_member_health_rule'] = (
+            utils.dict_from_parsed_args(parsed_args, [
+                "stack_id", "pool_id", "autoscaling_group_id"]))
+
         alarm['composite_rule'] = parsed_args.composite_rule
         if self.create:
             alarm['type'] = parsed_args.type
